from django.test import TestCase

from ..models import Message, MessageQuerySet


class MessageTests(TestCase):
    """
    Test the methods and custom manager of the Message model.

    """
    fixtures = ['soapboxtest.json']

    def test_global_instance(self):
        """
        Test a Message with is_global=True to ensure it matches on
        arbitrary URLs.

        """
        message = Message.objects.get(pk=1)
        self.assertTrue(
            message.match('/')
        )
        self.assertTrue(
            message.match('/foo/')
        )
        self.assertTrue(
            message.match('/foo/bar/')
        )

    def test_non_global_instance(self):
        """
        Test a message with is_global=False to ensure it matches only
        on the correct URL.

        """
        message = Message.objects.get(pk=2)
        self.assertFalse(
            message.match('/')
        )
        self.assertTrue(
            message.match('/foo/')
        )

    def test_prefix_matching(self):
        """
        Test that a Message matches when its URL is a prefix of the
        supplied URL.

        """
        message = Message.objects.get(pk=2)
        self.assertTrue(
            message.match('/foo/bar/')
        )

    def test_active_queryset(self):
        """
        Test that the active() method of MessageQuerySet returns the
        correct count.

        """
        self.assertEqual(
            MessageQuerySet(Message).active().count(),
            5
        )

    def test_active_manager(self):
        """
        Test that the active() method of MessageManager correctly
        passes through to MessageQuerySet.

        """
        self.assertEqual(
            Message.objects.active().count(),
            5
        )

    def test_match_global(self):
        """
        Test that the match() method of MessageManager correctly
        retrieves global Messages.

        """
        results = Message.objects.match('/')
        self.assertEqual(len(results), 1)
        result_ids = [m.id for m in results]
        self.assertEqual([1], result_ids)

    def test_match_simple(self):
        """
        Test that the match() method of MessageManager correctly
        retrieves global Messages and exact matches when both are
        present.

        """
        results = Message.objects.match('/bar/')
        self.assertEqual(len(results), 2)
        result_ids = [m.id for m in results]
        self.assertTrue(1 in result_ids)
        self.assertTrue(4 in result_ids)

    def test_match_partial(self):
        """
        Test that the match() method of MessageManager correctly
        retrieves Messages which match a prefix of the URL.

        """
        results = Message.objects.match('/foo/bar/')
        self.assertEqual(len(results), 4)
        result_ids = [m.id for m in results]
        self.assertTrue(1 in result_ids)
        self.assertTrue(2 in result_ids)
        self.assertTrue(3 in result_ids)
        self.assertTrue(7 in result_ids)


class ContextProcessorTests(TestCase):
    """
    Test the context processor which automatically adds messages to
    template context.

    """
    fixtures = ['soapboxtest.json']

    def test_context_processor(self):
        """
        Test basic use of the context processor.

        """
        # Enable the context processor only for this test, since its
        # context variable name conflicts with the one used by the
        # template tag in other tests.
        with self.modify_settings(
            TEMPLATE_CONTEXT_PROCESSORS={
                'append': 'soapbox.context_processors.soapbox_messages',
            }):
            r = self.client.get('/foo/bar/baz/')
            self.assertEqual(r.status_code, 200)
            self.assertTrue('soapbox_messages' in r.context)
            self.assertEqual(
                len(r.context['soapbox_messages']), 4)
            result_ids = [m.id for m in \
                          r.context['soapbox_messages']]
            self.assertTrue(1 in result_ids)
            self.assertTrue(2 in result_ids)
            self.assertTrue(3 in result_ids)
            self.assertTrue(7 in result_ids)
            



from django.contrib import admin
from django.utils.text import Truncator

from .models import Message


class MessageAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('url', 'message'),
        }),
        ('Options', {
            'fields': ('is_active', 'is_global'),
        }),
    )
    list_display = ('message_display', 'is_global', 'is_active', 'url')
    list_filter = ('is_global', 'is_active')

    def message_display(self, obj):
        """
        Truncate message to 50 characters.

        """
        return Truncator(obj.message).chars(50, html=True)
    message_display.allow_tags = True
    message_display.short_description = 'Message'


admin.site.register(Message, MessageAdmin)



from django import template
from django.test import TestCase


class TagTests(TestCase):
    fixtures = ['soapboxtest.json']

    def test_success_root(self):
        """
        Test retrieval on the root URL, which should retrieve only a
        global Message.

        """
        r = self.client.get('/')
        self.assertEqual(r.status_code, 200)
        self.assertEqual(
            len(r.context['soapbox_messages']), 1)
        self.assertContains(
            r, "This is a global message.")

    def test_success_with_match(self):
        """
        Test retrieval on a URL which will have a global Message and a
        Message with an exact URL match.

        """
        r = self.client.get('/foo/')
        self.assertEqual(r.status_code, 200)
        self.assertEqual(
            len(r.context['soapbox_messages']), 2)
        self.assertContains(
            r, "This is a global message.")
        self.assertContains(
            r, "This message appears on /foo/ and on /foo/bar/.")

    def test_success_with_multiple_match(self):
        """
        Test retrieval on a URL which will have a global Message, an
        exact URL match and a prefix URL match.

        """
        r = self.client.get('/foo/bar/')
        self.assertEqual(r.status_code, 200)
        self.assertEqual(
            len(r.context['soapbox_messages']), 4)
        self.assertContains(
            r, "This is a global message.")
        self.assertContains(
            r, "This message appears on /foo/ and on /foo/bar/.")
        self.assertContains(
            r, "This message appears only on /foo/bar/.")

    def test_html(self):
        """
        Test that HTML in Messages is passed through unescaped.

        """
        r = self.client.get('/foo/bar/')
        self.assertEqual(r.status_code, 200)
        self.assertContains(r, '<a href="/foo/bar/">')

    def test_fail_syntax(self):
        """
        Test that incorrect tag syntax raises TemplateSyntaxError.

        """
        self.assertRaises(
            template.TemplateSyntaxError,
            self.client.get,
            '/fail/')

    def test_bad_url_var(self):
        """
        Test that a bad URL variable passed to the tag bails out with
        no results.

        """
        r = self.client.get('/bad-url-var/')
        self.assertEqual(r.status_code, 200)
        self.assertTrue(
            'soapbox_messages' not in r.context
        )



from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.safestring import mark_safe


class MessageQuerySet(models.QuerySet):
    """
    Custom QuerySet implementing an active() method for only
    retrieving active Messages.

    """
    def active(self):
        return self.filter(is_active=True)


class MessageManager(models.Manager):
    """
    Custom manager using MessageQuerySet and implementing a query
    method for Messages which match a particular URL.

    """
    def get_queryset(self):
        return MessageQuerySet(self.model)

    def active(self):
        return self.get_queryset().active()

    def match(self, url):
        """
        Return a list of all active Messages which match the given
        URL.

        """
        results = set()
        for message in self.active():
            if message.is_global or message.match(url):
                results.add(message)
        return list(results)


@python_2_unicode_compatible
class Message(models.Model):
    """
    A message which may be displayed on some or all pages of a site.

    """
    message = models.TextField()
    is_global = models.BooleanField(
        default=False,
        help_text="If checked, this message will display on all pages."
    )
    is_active = models.BooleanField(
        default=True,
        help_text="Only active messages will be displayed."
    )
    url = models.CharField(
        "URL", max_length=255, blank=True, null=True,
        help_text="Message will be displayed on any URL which matches this."
    )

    objects = MessageManager()

    class Meta:
        ordering = ['-id']

    def __str__(self):
        return mark_safe(self.message)

    def match(self, url):
        """
        Determine whether this Message matches a given URL.

        The matching algorithm will return True if the Message is
        global, or if the Message's URL is a (case-sensitive) prefix
        of the supplied URL, and False otherwise.

        """
        if self.is_global:
            return True

        # For easy comparison, we strip leading and trailing slashes,
        # and then split both self.url and the supplied URL on
        # slashes, to get two lists of path components we can compare.
        self_bits = self.url.strip('/').split('/')
        url_bits = url.strip('/').split('/')

        # If self.url produced a longer list of path components than
        # the supplied URL, it can't be a match.
        if len(self_bits) > len(url_bits):
            return False

        return self_bits == url_bits[:len(self_bits)]



from django.conf.urls import patterns, url
from django.views.generic import TemplateView


urlpatterns = patterns(
    '',
    url(r'^$',
        TemplateView.as_view(
            template_name='soapboxtest/test_success.html')),
    url(r'^foo/$',
        TemplateView.as_view(
            template_name='soapboxtest/test_success.html')),
    url(r'^foo/bar/$',
        TemplateView.as_view(
            template_name='soapboxtest/test_success.html')),
    url(r'^foo/bar/baz/$',
        TemplateView.as_view(
            template_name='soapboxtest/test_context_processor.html')),
    url(r'^fail/$',
        TemplateView.as_view(
            template_name='soapboxtest/test_fail_syntax.html')),
    url(r'^bad-url-var/$',
        TemplateView.as_view(
            template_name='soapboxtest/test_bad_variable.html')),
)



from django import template

from ..models import Message


register = template.Library()


class MessagesForPageNode(template.Node):
    def __init__(self, url, varname):
        self.url = template.Variable(url)
        self.varname = varname

    def render(self, context):
        try:
            url = self.url.resolve(context)
        except template.VariableDoesNotExist:
            return ''
        context[self.varname] = Message.objects.match(url)
        return ''


@register.tag
def get_messages_for_page(parser, token):
    """
    Given a URL, return into the context a list of all
    :model:`soapbox.Message` objects matching that URL.

    Syntax::

        {% get_messages_for_page url as varname %}

    """
    bits = token.split_contents()
    if len(bits) != 4 or bits[2] != 'as':
        raise template.TemplateSyntaxError(
            "%s syntax must be '%s [url] as [varname]" % (
                bits[0], bits[0]))
    return MessagesForPageNode(bits[1], bits[3])



import os

from setuptools import setup


setup(name='django-soapbox',
      version='1.1',
      zip_safe=False, # eggs are the devil.
      description='Site-wide and page-specific announcements/messages for Django sites',
      long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
      author='James Bennett',
      author_email='james@b-list.org',
      url='https://github.com/ubernostrum/django-soapbox',
      packages=['soapbox', 'soapbox.templatetags', 'soapbox.tests', 'soapbox.templates'],
      package_dir={'soapbox': 'soapbox'},
      package_data={'soapbox': ['templates/soapboxtest/*.html']},
      test_suite='soapbox.runtests.run_tests',
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Framework :: Django :: 1.7',
                   'Framework :: Django :: 1.8',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3',
                   'Programming Language :: Python :: 3.3',
                   'Programming Language :: Python :: 3.4',
                   'Topic :: Utilities'],
      )



from .models import Message


def soapbox_messages(request):
    """
    Context processor which will retrieve all Message objects for the
    request URL and insert them into the context as the variable
    ``soapbox_messages``.

    """
    messages = Message.objects.match(request.path)
    return {'soapbox_messages': messages}
